#include "stdafx.h"
#include "FTStateManager.h"
#include "FTStateSend.h"
#include "FTStateRecv.h"
#include "UIChatFileElement.h"

namespace FileTransferState{
	//---------------------------------
	// implemenation of CFTStateManager
	// 

	CFTStateManager::CFTStateManager(UIChatFileElement * pUIElem)
		:m_pUIElem(pUIElem)
	{
		m_pState = NULL;
		m_state = FTS_NONE;
	}

	CFTStateManager::~CFTStateManager()
	{
		if(m_pState)
		{
			delete m_pState;
			m_pState = NULL;
		}
		m_pUIElem = NULL;
	}

	void CFTStateManager::SetState(_enum_file_transfer_state state)
	{
		CSingleLock lock(&m_csLock, TRUE);

		if(state == m_state)
			return;
		TRACE(TEXT("CFTStateManager::SetState, oldState=%d, newState=%d\n"), m_state, state);
		m_state = state;
		_UpdateState();

	}

	CSize CFTStateManager::CalcSize(const CPoint& ptPos, DWORD dwFlags,  CRect& rc)
	{
		CSingleLock lock(&m_csLock, TRUE);
		ASSERT(m_pState);
		if (m_pState)
		{
			return m_pState->CalcSize(ptPos, dwFlags, rc);
		}
		return CSize(0, 0);
	}

	BOOL CFTStateManager::DrawItems(Graphics *pGraph)
	{
		CSingleLock lock(&m_csLock, TRUE);
		ASSERT(m_pState);
		if (m_pState)
		{
			return m_pState->DrawItems(pGraph);
		}
		return FALSE;
	}

	void CFTStateManager::OnLButtonDown(UINT nFlags, CPoint point)
	{
		CSingleLock lock(&m_csLock, TRUE);
		ASSERT(m_pState);
		if (m_pState/* && m_pUIElem->IsEnable()*/)
		{
			m_pState->OnLButtonDown(nFlags, point);
		}
	}

	void CFTStateManager::OnLButtonUp(UINT nFlags, CPoint point)
	{
		CSingleLock lock(&m_csLock, TRUE);
		ASSERT(m_pState);
		if (m_pState/* && m_pUIElem->IsEnable()*/)
		{
			m_pState->OnLButtonUp(nFlags, point);
		}
	}

	BOOL CFTStateManager::OnMouseMove( UINT nFlags, CPoint point )
	{
		CSingleLock lock(&m_csLock, TRUE);
		ASSERT(m_pState);
		BOOL bHandled = FALSE;
		if (m_pState/* && m_pUIElem->IsEnableRetryButton()*/)
		{
			bHandled = m_pState->OnMouseMove(nFlags, point);
		}

		return bHandled;

	}


	void CFTStateManager::_UpdateState()
	{
		CSingleLock lock(&m_csLock, TRUE);

		delete m_pState;
		m_pState = NULL;

		switch(m_state)
		{
		case send_init:
			{
				m_pState = new CStateSendInit(m_pUIElem);
				ASSERT(m_pState);
				break;
			}
		case send_progress:
			{
				m_pState = new CStateSendProgress(m_pUIElem);
				ASSERT(m_pState);
				break;
			}
		case send_cancel:
			{
				m_pState = new CStateSendCancel(m_pUIElem);
				ASSERT(m_pState);
				break;
			}
		case send_complete:
			{
				m_pState = new CStateSendComplete(m_pUIElem);
				ASSERT(m_pState);
				break;
			}
		case send_error:
			{
				m_pState = new CStateSendError(m_pUIElem);
				ASSERT(m_pState);
				break;
			}
		case recv_init:
			{
				m_pState = new CStateRecvInit(m_pUIElem);
				break;
			}
		case recv_progress:
			{
				m_pState = new CStateRecvProgress(m_pUIElem);
				break;
			}
		case recv_cancel:
			{
				m_pState = new CStateRecvCancel(m_pUIElem);
				break;
			}
		case recv_error:
			{
				m_pState = new CStateRecvError(m_pUIElem);
				break;
			}
		case recv_complete:
			{
				m_pState = new CStateRecvComplete(m_pUIElem);
				break;
			}
		default:
			ASSERT(FALSE && TEXT("ERROR: CFTStateManager::_UpdateState()"));
			break;
		}
	}

}
